import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import "core-js/modules/es6.regexp.search";
import "core-js/modules/es6.function.name";
import _objectSpread from "@babel/runtime/helpers/objectSpread";

var _dec, _dec2, _class, _class2, _temp;

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React, { Component } from 'react';
import withStyles from '@material-ui/core/styles/withStyles';
import Button from '@material-ui/core/Button';
import Input from '@material-ui/core/Input';
import IconButton from '@material-ui/core/IconButton';
import SearchIcon from '@material-ui/icons/Search';
import SearchReset from '@material-ui/icons/Clear';
import Typography from '@material-ui/core/Typography';
import { types } from 'mobx-state-tree';
import PropTypes from 'prop-types';
import Highlight from 'react-highlighter';
import CircularProgress from '@material-ui/core/CircularProgress';
import Link from './Link';
import { PropTypes as MobxPropTypes, inject, observer } from 'mobx-react';
import Image from './Image';
/**
 * A search popup with suggestions of search texts, categories, products.
 * can be used like composition
 * <SearchPopup>
 *   <SuggestedSearch />
 *   <CategorySearches />
 *   <ProductSuggestions />
 * </SearchPopup>
 */

export var styles = function styles(theme) {
  return {
    searchPopup: {
      position: 'fixed',
      top: 0,
      bottom: 0,
      left: 0,
      right: 0,
      backgroundColor: 'rgba(255, 255, 255, .8)',
      zIndex: 9999,
      padding: '20px'
    },
    closeButton: {
      backgroundColor: theme.palette.white,
      border: "1px solid ".concat(theme.palette.border),
      '& span': {
        textTransform: 'uppercase',
        fontWeight: 'bold'
      }
    },
    searchForm: {
      display: 'flex',
      marginTop: '20px',
      position: 'relative'
    },
    searchReset: {
      position: 'absolute',
      top: 0,
      right: '50px'
    },
    flexInput: {
      flexGrow: 1,
      backgroundColor: theme.palette.secondary.light,
      boxSizing: 'border-box',
      height: 48,
      padding: '8px 12px 8px 12px'
    },
    searchButton: {
      backgroundColor: theme.palette.darkButton,
      borderRadius: 0,
      '& svg': {
        fill: theme.palette.white
      }
    },
    suggestionHeader: {
      textTransform: 'uppercase',
      marginTop: '40px'
    },
    suggestionsList: {
      listStyle: 'none',
      margin: 0,
      padding: 0,
      '& a strong': {
        fontWeight: 'bold',
        color: theme.palette.text.primary
      }
    },
    productImage: {
      height: '120px',
      width: '120px',
      display: 'block',
      marginBottom: '15px'
    },
    productsSuggestions: {
      display: 'flex',
      listStyle: 'none',
      margin: 0,
      padding: 0,
      overflowX: 'auto',
      '& > li': {
        margin: '5px'
      }
    },
    searchResultsWrapper: {
      position: 'absolute',
      top: '125px',
      bottom: 0,
      overflowY: 'auto',
      left: '20px',
      right: 0
    },
    loading: {
      paddingTop: '20%',
      display: 'flex',
      alignItems: 'center',
      justifyContent: 'center'
    }
  };
};
var SearchPopup = (_dec = withStyles(styles, {
  name: 'RSFSearchPopup'
}), _dec2 = inject(function (_ref) {
  var _ref$app = _ref.app,
      searchPopup = _ref$app.searchPopup,
      search = _ref$app.search,
      history = _ref.history;
  return {
    searchPopup: searchPopup,
    search: search,
    history: history
  };
}), _dec(_class = _dec2(_class = observer(_class = (_temp = _class2 =
/*#__PURE__*/
function (_Component) {
  _inherits(SearchPopup, _Component);

  function SearchPopup() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, SearchPopup);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(SearchPopup)).call.apply(_getPrototypeOf2, [this].concat(args)));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "state", {
      search: ''
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onChangeSearchText", function (text) {
      var search = _this.props.search;

      _this.setState({
        search: text
      }); // only apply loading spinner if there is some text
      // in case of input clearing we rejecting previous requests so they won't fire loading flag reset
      // that's why we're doing this here


      search.setLoading(!!text);
      search.run(text);
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "hidePopup", function () {
      var searchPopup = _this.props.searchPopup;
      searchPopup.setIsPopupOpened(false);

      _this.onChangeSearchText('');
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onSearchSubmit", function (event) {
      var history = _this.props.history;
      event.preventDefault();
      history.push("/search?text=".concat(_this.state.search));

      _this.hidePopup();
    });

    return _this;
  }

  _createClass(SearchPopup, [{
    key: "render",
    value: function render() {
      var _this2 = this;

      var _this$props = this.props,
          searchPopup = _this$props.searchPopup,
          classes = _this$props.classes,
          search = _this$props.search,
          children = _this$props.children,
          suggestionResultsLimit = _this$props.suggestionResultsLimit;

      var searches = _toConsumableArray(search.searches).splice(0, suggestionResultsLimit);

      var categories = _toConsumableArray(search.categories).splice(0, suggestionResultsLimit);

      var results = React.Children.map(children, function (list) {
        return React.cloneElement(list, {
          searches: searches,
          classes: classes,
          categories: categories,
          products: search.products,
          searchText: _this2.state.search,
          hidePopup: _this2.hidePopup
        });
      });
      var loading = search.loading;
      var contentReady = this.state.search && !loading;
      return React.createElement("form", {
        onSubmit: this.onSearchSubmit,
        onReset: function onReset() {
          _this2.onChangeSearchText('');
        }
      }, searchPopup.opened && React.createElement("div", {
        className: classes.searchPopup
      }, React.createElement(Button, {
        className: classes.closeButton,
        variant: "contained",
        onClick: this.hidePopup
      }, "Cancel"), React.createElement("div", {
        className: classes.searchForm
      }, React.createElement(Input, {
        type: "text",
        className: classes.flexInput,
        value: this.state.search,
        onChange: function onChange(e) {
          _this2.onChangeSearchText(e.target.value);
        }
      }), React.createElement(IconButton, {
        variant: "contained",
        type: "submit",
        className: classes.searchButton
      }, React.createElement(SearchIcon, null)), this.state.search && React.createElement(IconButton, {
        type: "reset",
        className: classes.searchReset
      }, React.createElement(SearchReset, null))), loading && React.createElement("div", {
        className: classes.loading
      }, React.createElement(CircularProgress, null)), contentReady && React.createElement("div", {
        className: classes.searchResultsWrapper
      }, results)));
    }
  }]);

  return SearchPopup;
}(Component), _defineProperty(_class2, "defaultProps", {
  suggestionResultsLimit: 3
}), _defineProperty(_class2, "propTypes", {
  /**
   * searchPopup holds state of popup
   */
  searchPopup: PropTypes.shape({
    opened: PropTypes.bool.isRequired
  }),

  /**
   * search contains lists of suggestions
   */
  search: PropTypes.shape({
    searches: MobxPropTypes.arrayOrObservableArrayOf(PropTypes.shape({
      text: PropTypes.string
    })),
    categories: MobxPropTypes.arrayOrObservableArrayOf(PropTypes.shape({
      id: PropTypes.string,
      url: PropTypes.string,
      name: PropTypes.string
    })),
    products: MobxPropTypes.arrayOrObservableArrayOf(PropTypes.shape({
      id: PropTypes.string,
      url: PropTypes.string,
      name: PropTypes.string,
      image: PropTypes.string
    }))
  }),

  /**
   * limit for suggestion results list
   */
  suggestionResultsLimit: PropTypes.number,

  /**
   * Suggestion results components
   */
  children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node]).isRequired,
  history: PropTypes.shape({
    push: PropTypes.func.isRequired
  })
}), _temp)) || _class) || _class) || _class);
export { SearchPopup as default };
var commonPropTypes = {
  hidePopup: PropTypes.func,
  classes: PropTypes.shape({
    suggestionHeader: PropTypes.string,
    suggestionsList: PropTypes.string
  })
  /**
   * Search text suggestions
   * @param props
   * @constructor
   */

};
export var SuggestedSearch = function SuggestedSearch(props) {
  return React.createElement(React.Fragment, null, React.createElement(Typography, {
    component: "h2",
    className: props.classes.suggestionHeader
  }, "Suggested search"), React.createElement("ul", {
    className: props.classes.suggestionsList
  }, props.searches.map(function (item, i) {
    return React.createElement("li", {
      key: i
    }, React.createElement(Link, {
      to: "/search?text=".concat(item.text),
      onClick: props.hidePopup
    }, React.createElement(Highlight, {
      search: props.searchText
    }, item.text)));
  })));
};
SuggestedSearch.propTypes = _objectSpread({}, commonPropTypes, {
  searches: MobxPropTypes.arrayOrObservableArrayOf(PropTypes.shape({
    text: PropTypes.string
  })),
  searchText: PropTypes.string
  /**
   * Search categories suggestions
   * @param props
   * @constructor
   */

});
export var CategorySearches = function CategorySearches(props) {
  return React.createElement(React.Fragment, null, React.createElement(Typography, {
    component: "h2",
    className: props.classes.suggestionHeader
  }, "Category searches"), React.createElement("ul", {
    className: props.classes.suggestionsList
  }, props.categories.map(function (item) {
    return React.createElement("li", {
      key: item.id
    }, React.createElement(Link, {
      to: item.url,
      onClick: props.hidePopup
    }, item.name));
  })));
};
CategorySearches.propTypes = _objectSpread({}, commonPropTypes, {
  categories: MobxPropTypes.arrayOrObservableArrayOf(PropTypes.shape({
    id: PropTypes.string,
    url: PropTypes.string,
    name: PropTypes.string
  }))
  /**
   * Search products suggestions
   * @param props
   * @constructor
   */

});
export var ProductSuggestions = function ProductSuggestions(props) {
  return React.createElement(React.Fragment, null, React.createElement(Typography, {
    component: "h2",
    className: props.classes.suggestionHeader
  }, "Product suggestions"), React.createElement("ul", {
    className: props.classes.productsSuggestions
  }, props.products.map(function (item) {
    return React.createElement("li", {
      key: item.id
    }, React.createElement(Link, {
      to: item.url,
      onClick: props.hidePopup,
      className: props.classes.productImage
    }, React.createElement(Image, {
      fill: true,
      src: item.thumbnail,
      alt: item.name
    })), React.createElement(Link, {
      to: item.url,
      onClick: props.hidePopup
    }, item.name));
  })));
};
ProductSuggestions.propTypes = _objectSpread({}, commonPropTypes, {
  products: MobxPropTypes.arrayOrObservableArrayOf(PropTypes.shape({
    id: PropTypes.string,
    url: PropTypes.string,
    name: PropTypes.string,
    thumbnail: PropTypes.string
  }))
  /**
   * Search popup model
   */

});
export var SearchPopupModel = types.model('SearchPopupModel', {
  opened: types.boolean
}).actions(function (self) {
  return {
    setIsPopupOpened: function setIsPopupOpened(status) {
      self.opened = status;
      return self;
    }
  };
});
//# sourceMappingURL=SearchPopup.js.map