/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React from 'react';
import { hydrate } from './renderers';
import createBrowserHistory from 'history/createBrowserHistory';
import registerServiceWorker from './registerServiceWorker';
import PWA from './PWA';
/**
 * Bootstraps the PWA react application.
 * @param {options} options
 * @param {React.Element} options.App The root app element
 * @param {Object} options.theme A material-ui theme
 * @param {Class} options.model A mobx-state-tree model class, typically extending `react-storefront/model/AppModelBase`
 * @param {HTMLElement} options.target The DOM element to mount onto
 * @param {Function} options.errorReporter A function to call when an error occurs so that it can be logged, typically located in `src/errorReporter.js`.
 */

export default function launchClient(_ref) {
  var App = _ref.App,
      theme = _ref.theme,
      model = _ref.model,
      router = _ref.router,
      _ref$target = _ref.target,
      target = _ref$target === void 0 ? document.getElementById('root') : _ref$target,
      _ref$errorReporter = _ref.errorReporter,
      errorReporter = _ref$errorReporter === void 0 ? Function.prototype : _ref$errorReporter;
  var history = createBrowserHistory();
  hydrate({
    component: React.createElement(PWA, {
      errorReporter: errorReporter
    }, React.createElement(App, null)),
    model: model,
    theme: theme,
    target: target,
    providerProps: {
      history: history,
      router: router
    }
  });
  registerServiceWorker();
}
//# sourceMappingURL=launchClient.js.map